@@ -6,8 +6,9 @@ import tempfile
 import lastfm.config
 import lastfm.marshaller
 
-SLEEP_TIME = 5
-PIDFILE_BASE = '/var/run/lastfm'
+SLEEP = 5
+PID = lastfm.config.DefaultPath(lambda n: '/var/run/lastfm/%s.pid' % n,
+                                lambda n: '~/.%s/pid' % n)
 
 class Client:
     """Something that uses the lastfmsubmitd spool. Has a configuration
@@ -22,10 +23,7 @@ class Client:
         config, but if ``conf`` is supplied, talk to that one."""
 
         self.name = name
-        if conf:
-            self.conf = conf
-        else:
-            self.conf = lastfm.config.Config(search='lastfmsubmitd')
+        self.conf = conf or lastfm.config.Config()
 
     def open_log(self, debug=False, stderr=False):
         """Returns a logging object that will write to the client's log. If
@@ -86,28 +84,37 @@ class Daemon(Client):
     def __init__(self, name, conf=None, log=True):
         Client.__init__(self, name, conf, log)
         self.conf.sleep_time = float(self.conf.cp.get('daemon', 'sleep_time',
-            SLEEP_TIME))
+            SLEEP))
         self.conf.pidfile_path = self.conf.cp.get('paths', 'pidfile',
-            '%s/%s.pid' % (PIDFILE_BASE, self.name))
+            self.conf.get_path(PID))
+
+    def fork(self):
+        try:
+            pid = os.fork()
+            if pid:
+                sys.exit(0)
+        except OSError, e:
+            print >>sys.stderr, "%s: can't fork: %s" % (self.name, e)
+            sys.exit(1)
 
     def daemonize(self, fork=True):
         if fork:
-            try:
-                pid = os.fork()
-                if pid: sys.exit(0)
-            except OSError, e:
-                print >>sys.stderr, "%s: can't fork: %s" % (self.name, e)
-                sys.exit(1)
+            self.fork()
+            if os.geteuid() == 0:
+                os.setsid()
+                self.fork()
 
-        os.chdir('/')
-        os.setsid()
-        os.umask(0)
+            os.chdir('/')
+            os.umask(0)
 
-        devnull = os.open('/dev/null', os.O_RDWR)
-        os.dup2(devnull, sys.stdin.fileno())
-        os.dup2(devnull, sys.stdout.fileno())
-        os.dup2(devnull, sys.stderr.fileno())
-        os.close(devnull)
+            devnull = os.open('/dev/null', os.O_RDWR)
+            os.dup2(devnull, sys.stdin.fileno())
+            os.dup2(devnull, sys.stdout.fileno())
+            os.dup2(devnull, sys.stderr.fileno())
+            os.close(devnull)
+
+        if not os.path.exists(self.conf.spool_path):
+            os.mkdir(self.conf.spool_path)
 
         try:
             pidfile = open(self.conf.pidfile_path, 'w')
